var zoomManager =
	{
		is_zoomed: false,
		is_unzooming: false,
		
		default_scale_factors:
			{
				start: 1.5,
				min: 1.25,
				max: 3.5
			},
		
		zoom_transition_duration: 750,
		friction_factor: 0.3,
		slide_overflow:
			{
				w: null,
				h: null
			},
		
		current_scale_factor: 1,
		pinching: false,
		
		central_point:
			{
				x: 0,
				y: 0
			},
		
		central_point_start:
			{
				x: 0,
				y: 0
			},
		
		central_point_previous:
			{
				x: null,
				y: null,
				timestamp: null
			},
		
		central_point_before_previous:
			{
				x: null,
				y: null,
				timestamp: null
			},
		
		scroll_start:
			{
				x: null,
				y: null
			},
		
		scroll_end:
			{
				x: null,
				y: null
			},
		
		scroll_boundaries:
			{
				top: null,
				right: null,
				bottom: null,
				left: null,
			},
		
		offset_left:
			{
				unzoomed: null,
				zoomed: null
			},
		
		offset_top:
			{
				unzoomed: null,
				zoomed: null
			},
		
		touches: [],
		
		velocity:
			{
				axis_x:
					{
						previous: 0,
						current: 0
					},
				axis_y:
					{
						previous: 0,
						current: 0
					}
			},
		
		acceleration: {
			axis_x: null,
			axis_y: null
		},
		
		pinch_start:
			{
				center_x: null,
				center_y: null,
				percent_x: null,
				percent_y: null,
				scale_factor: null
			},
		
		updateContent: function(nr_card)
			{
				var image_sources = this.getImagesSourcesFromCardNumber(nr_card);
				
				for (var i = 0; i < image_sources.length; i++)
					{
						$('#img_zoom_' + i).attr('src', image_sources[i]);
					}
			},
		
		getImagesSourcesFromCardNumber: function(nr_card)
			{
				var result = new Array(2);
				
				var obj_image = new Image();
				var image_index_start = (cardManager.nr_pages_per_card == 1) ? nr_card : (nr_card - 1) * 2;
				
				for (var i = 0; i < cardManager.nr_pages_per_card; i++)
					{
						var image_number = image_index_start + i;
						
						result[i] = ((image_number > 0) && (image_number <= cardManager.nr_pages))
													? app.path + 'Zoom/page' + image_number + '.jpg'
													: imagesManager.image_blank;
													
						obj_image.src = result[i];
					}
				
				if (cardManager.nr_pages_per_card == 1)
					{
						result[1] = imagesManager.image_blank;
					}
				
				return result;
			},
		
		showZoomPanel: function()
			{
				$('#zoom_panel').removeClass('zoom_panel_invisible')
												.css('-webkit-transform-origin', '0px 0px');
			},
		
		hideZoomPanel: function()
			{
				$('#zoom_panel').addClass('zoom_panel_invisible');
			},
		
		calculateOffsets: function()
			{
				this.offset_left.zoomed = parseInt((proportionsManager.view_dimensions.w * (1 - this.current_scale_factor) * cardManager.nr_pages_per_card / 2) / this.current_scale_factor);
				this.offset_top.zoomed = parseInt((proportionsManager.view_dimensions.h * (1 - this.current_scale_factor) / 2) / this.current_scale_factor);
			},
		
		calculateScrollBoundaries: function()
			{
				this.scroll_boundaries.left = ((this.offset_left.unzoomed + this.offset_left.zoomed) > 0) ? this.offset_left.zoomed : - this.offset_left.unzoomed;
				this.scroll_boundaries.right = this.offset_left.zoomed * 2 - this.scroll_boundaries.left;
				
				this.scroll_boundaries.top = ((this.offset_top.unzoomed + this.offset_top.zoomed) > 0) ? this.offset_top.zoomed : - this.offset_top.unzoomed;
				this.scroll_boundaries.bottom = this.offset_top.zoomed * 2 - this.scroll_boundaries.top;
			},
		
		checkScrollBoundariesExceeded: function()
			{
				var result = false;
				
				if (this.scroll_end.x < this.scroll_boundaries.right)
					{
						result = true;
						this.scroll_end.x = this.scroll_boundaries.right;
					}
				else if (this.scroll_end.x > this.scroll_boundaries.left)
					{
						result = true;
						this.scroll_end.x = this.scroll_boundaries.left;
					}
				
				if (this.scroll_end.y < this.scroll_boundaries.bottom)
					{
						result = true;
						this.scroll_end.y = this.scroll_boundaries.bottom;
					}
				else if (this.scroll_end.y > this.scroll_boundaries.top)
					{
						result = true;
						this.scroll_end.y = this.scroll_boundaries.top;
					}
				
				return result;
			},
		
		limitScrollOverflowX: function()
			{
				if (this.scroll_end.x < this.scroll_boundaries.right)
					{
						var diff = parseInt((this.scroll_end.x - this.scroll_boundaries.right) * this.current_scale_factor);
						
						if (Math.abs(diff) > this.slide_overflow.w)
							{
								this.scroll_end.x = parseInt((this.scroll_boundaries.right - this.slide_overflow.w / this.current_scale_factor));
							}
					}
				else if (this.scroll_end.x > this.scroll_boundaries.left)
					{
						var diff = parseInt((this.scroll_end.x - this.scroll_boundaries.left) * this.current_scale_factor);
						
						if (Math.abs(diff) > this.slide_overflow.w)
							{
								this.scroll_end.x = parseInt((this.scroll_boundaries.left + this.slide_overflow.w / this.current_scale_factor));
							}
					}
			},
		
		limitScrollOverflowY: function()
			{
				if (this.scroll_end.y < this.scroll_boundaries.bottom)
					{
						var diff = parseInt((this.scroll_end.y - this.scroll_boundaries.bottom) * this.current_scale_factor);
						
						if (Math.abs(diff) > this.slide_overflow.h)
							{
								this.scroll_end.y = parseInt((this.scroll_boundaries.bottom - this.slide_overflow.h / this.current_scale_factor));
							}
					}
				else if (this.scroll_end.y > this.scroll_boundaries.top)
					{
						var diff = parseInt((this.scroll_end.y - this.scroll_boundaries.top) * this.current_scale_factor);
						
						if (Math.abs(diff) > this.slide_overflow.h)
							{
								this.scroll_end.y = parseInt((this.scroll_boundaries.top + this.slide_overflow.h / this.current_scale_factor));
							}
					}
			},
		
		zoomElement: function(center_x, center_y, scale_factor, has_duration)
			{
				has_duration = (has_duration !== false) ? true : has_duration;
				var duration = 0;
				
				if (has_duration)
					{
						duration = this.zoom_transition_duration;
					}
				
				visualComponents.pages_panel.hide();
				this.showZoomPanel();
				componentsManager.hideTopBar();
				componentsManager.hideBottomBar();
				
				this.current_scale_factor = (scale_factor == undefined)
																			? this.default_scale_factors.start
																			: scale_factor;
				
				windowUrl.changeHash('state', 'zoom');
				
				this.calculateOffsets();
				
				this.scroll_end.x = (center_x == undefined)
															? this.offset_left.zoomed
															: parseInt(center_x * (1 - this.current_scale_factor) / this.current_scale_factor);
				
				this.scroll_end.y = (center_y == undefined)
															? this.offset_top.zoomed
															: parseInt(center_y * (1 - this.current_scale_factor) / this.current_scale_factor);
				
				this.calculateScrollBoundaries();
				this.checkScrollBoundariesExceeded();
				
				this.scroll_start.x = this.scroll_end.x;
				this.scroll_start.y = this.scroll_end.y;
				
				$('#zoom_container_scaled').css('-webkit-transition-duration', duration + 'ms')
																	 .css('-webkit-transition-timing-function', 'ease-out');
				
				var self = this;
				
				setTimeout(function()
					{
						$('#zoom_container_scaled').css('-webkit-transform', 'scale(' + self.current_scale_factor + ') translate(' + self.scroll_end.x + 'px, ' + self.scroll_end.y + 'px)');
						
						self.is_zoomed = true;
					},
					10
				);
			},
		
		unZoomElement: function(has_duration)
			{
				this.is_unzooming = true;
				
				has_duration = (has_duration !== false) ? true : has_duration;
				var duration = 0;
				
				if (has_duration)
					{
						duration = this.zoom_transition_duration;
					}
				
				this.current_scale_factor = 1;
				
				windowUrl.changeHash('state', '');
				
				this.scroll_end.x = 0;
				this.scroll_end.y = 0;
				
				var self = this;
				
				$('#zoom_container_scaled').css('-webkit-transition-duration', duration + 'ms')
																	 .css('-webkit-transition-timing-function', 'ease-in');
				
				setTimeout(function()
					{
						$('#zoom_container_scaled').css('-webkit-transform', 'scale(' + self.current_scale_factor + ') translate(' + self.scroll_end.x + 'px, ' + self.scroll_end.y + 'px)');
						
						self.is_zoomed = false;
						
						setTimeout(function()
							{
								zoomManager.hideZoomPanel();
								visualComponents.pages_panel.show();
								self.is_unzooming = false;
							},
							duration
						);
					},
					10
				);
			},
		
		rescalingBegin: function(touches)
			{
				this.pinching = true;
				
				this.pinch_start.percent_x = this.central_point.x / (proportionsManager.stage.w * this.current_scale_factor);
				this.pinch_start.percent_y = this.central_point.y / (proportionsManager.stage.h * this.current_scale_factor);
				
				this.pinch_start.scale_factor = this.current_scale_factor;
			},
		
		rescalingEnd: function()
			{
				this.pinching = false;
				
				var delta_x = this.central_point.x - this.central_point_previous.x;
				var delta_y = this.central_point.y - this.central_point_previous.y;
				
				this.central_point_start.x += delta_x;
				this.central_point_start.y += delta_y;
				
				this.scroll_start.x = this.scroll_end.x;
				this.scroll_start.y = this.scroll_end.y;
			},
		
		rescaleElement: function(touches, scale_factor, timestamp)
			{
				if (!this.is_unzooming)
					{
						this.current_scale_factor = this.pinch_start.scale_factor * scale_factor;
						
						if (this.current_scale_factor > this.default_scale_factors.max)
							{
								this.current_scale_factor = this.default_scale_factors.max;
							}
						else if (this.current_scale_factor <= this.default_scale_factors.min)
							{
								this.current_scale_factor = this.default_scale_factors.min;
								
								this.unZoomElement();
								
								return;
							}
						
						this.calculateOffsets();
						
						var percent_x = this.central_point.x / (proportionsManager.stage.w * this.current_scale_factor);
						var percent_y = this.central_point.y / (proportionsManager.stage.h * this.current_scale_factor);
						
						var offset_x = parseInt((percent_x - this.pinch_start.percent_x) * proportionsManager.stage.w);
						var offset_y = parseInt((percent_y - this.pinch_start.percent_y) * proportionsManager.stage.h);
						
						this.scroll_end.x = this.scroll_start.x + offset_x;
						this.scroll_end.y = this.scroll_start.y + offset_y;
						
						this.calculateScrollBoundaries();
						this.checkScrollBoundariesExceeded();
						
						$('#zoom_container_scaled').css('-webkit-transition-duration', '100ms')
																			 .css('-webkit-transform', 'scale(' + this.current_scale_factor + ') translate(' + this.scroll_end.x + 'px, ' + this.scroll_end.y + 'px)');
					}
			},
		
		calculateCentralPoint: function(timestamp)
			{
				var sum_x = 0;
				var sum_y = 0;
				var counter = 0;
				
				for (var i = 0; i < this.touches.length; i++)
					{
						if (i <= 1)
							{
								sum_x += this.touches[i].pageX;
								sum_y += this.touches[i].pageY;
								counter += 1;
							}
						else
							{
								break;
							}
					}
				
				if (counter > 0)
					{
						this.central_point.x = parseInt(sum_x / counter);
						this.central_point.y = parseInt(sum_y / counter);
					}
			},
		
		translationBegin: function(touches, timestamp)
			{
				this.touches = touches;
				
				if (this.touches.length >= 2)
					{
						this.scroll_start.x = this.scroll_end.x;
						this.scroll_start.y = this.scroll_end.y;
					}
				
				this.calculateCentralPoint(timestamp);
				
				this.central_point_start.x = this.central_point.x;
				this.central_point_start.y = this.central_point.y;
				
				this.central_point_previous.x = this.central_point.x;
				this.central_point_previous.y = this.central_point.y;
				this.central_point_previous.timestamp = timestamp;
				
				this.central_point_before_previous.x = this.central_point_previous.x;
				this.central_point_before_previous.y = this.central_point_previous.y;
				this.central_point_before_previous.timestamp = this.central_point_previous.timestamp;
			},
		
		translationEnd: function(touch_identifier, timestamp)
			{
				if (!this.is_unzooming)
					{
						for (var i = 0; i < this.touches.length; i++)
							{
								if (this.touches[i].identifier == touch_identifier)
									{
										arrayUtils.removeByIndex(this.touches, i);
										
										break;
									}
							}
						
						this.calculateCentralPoint(timestamp);
						
						if (!this.pinching)
							{
								var scroll_x = this.scroll_end.x;
								var scroll_y = this.scroll_end.y;
								
								this.checkScrollBoundariesExceeded();
								
								if ((this.scroll_end.x == scroll_x) &&
										(scroll_x > this.scroll_boundaries.right) &&
										(scroll_x < this.scroll_boundaries.left))
									{
										var diff_x = this.central_point_previous.x - this.central_point_before_previous.x;
										
										if ((diff_x != undefined) && 
												(diff_x != 0))
											{
												this.acceleration.axis_x = parseInt(((this.acceleration.axis_x <= 100) ? this.acceleration.axis_x : 100) / 2);
												this.scroll_end.x += parseInt(this.acceleration.axis_x * (diff_x / Math.abs(diff_x)) * (1 / this.friction_factor));
												
												this.limitScrollOverflowX();
											}
									}
								
								if ((this.scroll_end.y == scroll_y) &&
										(scroll_y > this.scroll_boundaries.bottom) &&
										(scroll_y < this.scroll_boundaries.top))
									{
										var diff_y = this.central_point_previous.y - this.central_point_before_previous.y;
										
										if ((diff_y != undefined) &&
												(diff_y != 0))
											{
												this.acceleration.axis_y = parseInt(((this.acceleration.axis_y <= 100) ? this.acceleration.axis_y : 100) / 2);
												this.scroll_end.y += parseInt(this.acceleration.axis_y * (diff_y / Math.abs(diff_y)) * (1 / this.friction_factor));
												
												this.limitScrollOverflowY();
											}
									}
								
								var slide_duration = parseInt(200 / this.friction_factor);
								
								$('#zoom_container_scaled').css('-webkit-transition-duration', slide_duration + 'ms')
																					 .css('-webkit-transform', 'scale(' + this.current_scale_factor + ') translate(' + this.scroll_end.x + 'px, ' + this.scroll_end.y + 'px)');
								
								if (this.checkScrollBoundariesExceeded())
									{
										var self = this;
										
										setTimeout(function()
											{
												$('#zoom_container_scaled').css('-webkit-transition-duration', parseInt(slide_duration * 0.75) + 'ms')
																									 .css('-webkit-transform', 'scale(' + self.current_scale_factor + ') translate(' + self.scroll_end.x + 'px, ' + self.scroll_end.y + 'px)');
											},
											slide_duration
										);
									}
								
								this.scroll_start.x = this.scroll_end.x;
								this.scroll_start.y = this.scroll_end.y;
							}
					}
			},
		
		translateElement: function(touches, timestamp)
			{
				if (!this.is_unzooming)
					{
						this.touches = touches;
						
						this.calculateCentralPoint(timestamp);
						
						this.velocity.axis_x.previous = this.velocity.axis_x.current;
						this.velocity.axis_y.previous = this.velocity.axis_y.current;
						
						var diff_x = this.central_point.x - this.central_point_start.x;
						var diff_y = this.central_point.y - this.central_point_start.y;
						
						var diff_timestamp = timestamp - this.central_point_previous.timestamp;
						
						this.velocity.axis_x.current = Math.abs(diff_x) / (diff_timestamp * this.current_scale_factor);
						this.velocity.axis_y.current = Math.abs(diff_y) / (diff_timestamp * this.current_scale_factor);
						
						this.acceleration.axis_x = Math.abs((this.velocity.axis_x.current - this.velocity.axis_x.previous) / diff_timestamp) * 1000;
						this.acceleration.axis_y = Math.abs((this.velocity.axis_y.current - this.velocity.axis_y.previous) / diff_timestamp) * 1000;
						
						this.scroll_end.x = parseInt(diff_x / this.current_scale_factor) + this.scroll_start.x;
						this.scroll_end.y = parseInt(diff_y / this.current_scale_factor) + this.scroll_start.y;
						
						this.checkScrollBoundariesExceeded();
						
						$('#zoom_container_scaled').css('-webkit-transition-duration', '0ms')
																			 .css('-webkit-transform', 'scale(' + this.current_scale_factor + ') translate(' + this.scroll_end.x + 'px, ' + this.scroll_end.y + 'px)');
						
						this.central_point_before_previous.x = this.central_point_previous.x;
						this.central_point_before_previous.y = this.central_point_previous.y;
						this.central_point_before_previous.timestamp = this.central_point_previous.timestamp;
						
						this.central_point_previous.x = this.central_point.x;
						this.central_point_previous.y = this.central_point.y;
						this.central_point_previous.timestamp = timestamp;
					}
			}
	};